﻿Imports Tzu_Engine
Imports Tzu_Engine.TZGlobal
Imports Tzu_Engine.Mesh
Imports Tzu_Engine.Scene
Imports Tzu_Engine.Texture
Imports Tzu_Engine.Graphics
Imports Tzu_Engine.Shader

Public Class Form1

    Private Engine As TZEngine = TZEngine.GetInstance
    Private Input As TZInput = TZInput.GetInstance

    Private myCamera As TZCamera
    Private Camera_Speed_Def As Single = 6
    Private Camera_Speed As Single
    Private MoveDir As Integer = 1
    'Private Camera_Rotate As Boolean = True
    'Private Camera_MouseState As Vector3

    Private floor_mesh As TZMesh
    Private sphere_mesh(,,) As TZMesh

    Private myScene As TZScene

    Private meshManag As Mesh.TZMeshManager = Mesh.TZMeshManager.GetInstance

    Private LightManag As Light.TZLightManager = Light.TZLightManager.GetInstance

    Private myLightDir As Light.TZLight

    Private Occlusion_Enable As Boolean

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If Engine.DoRender = True Then
            e.Cancel = True
            Engine.Quit()
        End If
    End Sub

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Engine.Init(Me.Handle)       

        Engine.Display_FPS = True
        Engine.Add_MediaPath(My.Application.Info.DirectoryPath & "\..\..\..\media\")

        Engine.Init_Occlusion(128, 128)

        Create_Scene()

        myScene.Load()

        Engine.WaitLoad(False, True, True)

        Update_Scene()

        Render_Scene()

    End Sub

    Private Sub Create_Scene()



        myScene = New TZScene("myScene")

        'mesh
        floor_mesh = meshManag.Load_Cube(8, 8, 8, "gros_cube")
        floor_mesh.Material(0).Ambient = Color.Gray
        floor_mesh.Material(0).Diffuse = Color.Green
        floor_mesh.Material(0).Specular = Color.Gray
        floor_mesh.Material(0).SpecularSharpness = 45
        floor_mesh.AutoScale_Size = 15
        floor_mesh.AutoScale = True

        myScene.Add(floor_mesh)

        ReDim sphere_mesh(8, 8, 8)
        For i As Integer = 0 To sphere_mesh.GetUpperBound(0)
            For j As Integer = 0 To sphere_mesh.GetUpperBound(1)
                For k As Integer = 0 To sphere_mesh.GetUpperBound(2)
                    sphere_mesh(i, j, k) = meshManag.Load_Sphere(0.5, 128, 128, "mini_sphere" & i & "-" & j & "-" & k)
                    sphere_mesh(i, j, k).Position = New Vector3(10 + i * 2, -11 + j * 2, -8 + k * 2)
                    sphere_mesh(i, j, k).Material(0).Diffuse = Color.Red
                    sphere_mesh(i, j, k).IsOccluder = False
                    myScene.Add(sphere_mesh(i, j, k))
                Next
            Next
        Next


        myCamera = New TZCamera("myCamera")
        myCamera.Set_Mode(eCameraMode.First_Person)
        myCamera.Position = New Vector3(-15, -3, 0)
        myCamera.Perspective(75, CSng(Engine.Viewport_Width / Engine.Viewport_Height), 0.1, 50)
        myCamera.LookAt(New Vector3(0, -3, 0))
        LightManag.Lightning = True

        myLightDir = New Light.TZLight("mylight_directional")
        LightManag.Create_DirectionalLight(New Vector3(1, -0.5, 0.2), myLightDir)
        myLightDir.Ambient = Color.Gray
        myLightDir.Diffuse = Color.White
        myLightDir.Specular = Color.Gray
        myLightDir.Enable = True

        Me.Text = "Appuyez sur C pour activer l'Occlusion Culling "
    End Sub

    Private Sub Update_Scene()
        floor_mesh.Position = New Vector3(0, -floor_mesh.Bounding_Box.Height / 2, 0)
    End Sub

    Private Sub Render_Scene()


        Dim skycolor As Color = Get_Color(0.6, 0.8, 1, 1)
        Do While Engine.DoRender

            If Engine.TestCooperativeLevel Then

                User_Input()

                If Not Input.Key(eKeyboard.Space) Then
                    Move_Cam()
                End If

                Engine.Camera_Update(myCamera, True)

                If Occlusion_Enable Then
                    myScene.Check_Occlusion()
                End If

                Engine.Render_Clear(skycolor)

                Engine.Render_Begin()
                myScene.Render()
                Engine.Render_End()

            End If
            Application.DoEvents()
        Loop

        Engine.Dispose()

        Application.Exit()
    End Sub

    Private Sub Move_Cam()
        Camera_Speed = Camera_Speed_Def * Engine.DeltaTime

        If myCamera.Position.Z >= 6 Then
            MoveDir = -1
        ElseIf myCamera.Position.Z <= -6 Then
            MoveDir = 1
        End If

        myCamera.Move(New Vector3(0, 0, MoveDir), New Vector3(0, 0, Camera_Speed))
        
    End Sub

    Private Sub User_Input()
        If Input.Key(eKeyboard.Echap) Then Engine.Quit()

        If Input.Key_Pressed(eKeyboard.Grave, True) Then
            Engine.Display_Console = Not Engine.Display_Console
        End If
        If Input.Key_Pressed(eKeyboard.F1) Then
            Engine.Display_FPS = Not Engine.Display_FPS
        ElseIf Input.Key_Pressed(eKeyboard.F2) Then
            Engine.Render_BoundingBox = Not Engine.Render_BoundingBox
        ElseIf Input.Key_Pressed(eKeyboard.F3) Then
            myScene.Render_BoundingBox = Not myScene.Render_BoundingBox
        End If
        If Engine.Display_Console = True Then
            Engine.Console_Write = Input.Key_Pressed

            If Input.Key(eKeyboard.Home) Then
                Engine.Console_Maximize()
            ElseIf Input.Key(eKeyboard.Ends) Then
                Engine.Console_Normal()
            End If

            If Input.Key(eKeyboard.Page_Up) Then
                Engine.Console_ScrollLine -= 1
            ElseIf Input.Key(eKeyboard.Page_Down) Then
                Engine.Console_ScrollLine += 1
            End If

        Else
           
            If Input.Key_Pressed(eKeyboard.C) Then
                Occlusion_Enable = Not Occlusion_Enable
            End If

        End If

    End Sub


End Class
