﻿'Dans ce quatrième tutorial nous allons créer un terrain a partir d'une texture
'heightmap
Imports Tzu_Engine
Imports Tzu_Engine.TZGlobal

Public Class Form1

    Private Engine As TZEngine = TZEngine.GetInstance

    Private myCamera As Scene.TZCamera

    Private myTerrain As Terrain.TZTerrain

    Private mySkybox As Sky.TZSkybox

    Private RState As RenderState.TZRenderState = RenderState.TZRenderState.GetInstance

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialisation du moteur en mode fenêtré
        Engine.Init(Me.Handle)        

        'indique que nous voulons afficher le nombre d'image par seconde
        Engine.Display_FPS = True

        'on indique le dossier des medias       
        Engine.Add_MediaPath(My.Application.Info.DirectoryPath & "\..\..\..\media\", True)


        'on créé notre terrain
        myTerrain = New Terrain.TZTerrain("montagneheight.jpg", "myterrain", 6, 64, 1)      


        'On défini la texture de base qui est étiré sur tout le terrain
        myTerrain.Texture_Base = Texture.TZTextureManager.GetInstance.Load("montagne.jpg", "terrain_texture_black")

        'on ajoute la texture detail qui va ajouter un peu "bruit"
        myTerrain.Texture_Detail = Texture.TZTextureManager.GetInstance.Load("detail.jpg", "mydetailtex")

        'on tile la texture de detail pour qu'elle ce repete 64x64
        myTerrain.Tile_Texture_Detail = 64.0F

        'charge le terrain
        myTerrain.load()

        'ajout d'une skybox (cube englobant la camera dont chaque face est texturé)
        mySkybox = New Sky.TZSkybox("myskybox")        
        mySkybox.Texture(eSkybox.Back) = Texture.TZTextureManager.GetInstance.Load("skybox_back.jpg", "sky_back")
        mySkybox.Texture(eSkybox.Down) = Texture.TZTextureManager.GetInstance.Load("skybox_bottom.jpg", "sky_down")
        mySkybox.Texture(eSkybox.Front) = Texture.TZTextureManager.GetInstance.Load("skybox_front.jpg", "sky_front")
        mySkybox.Texture(eSkybox.Left) = Texture.TZTextureManager.GetInstance.Load("skybox_left.jpg", "sky_left")
        mySkybox.Texture(eSkybox.Right) = Texture.TZTextureManager.GetInstance.Load("skybox_right.jpg", "sky_right")
        mySkybox.Texture(eSkybox.Up) = Texture.TZTextureManager.GetInstance.Load("skybox_top.jpg", "sky_up")

        mySkybox.load()

        'ajout d'un brouillard bleu ciel
        RState.Fog_Density = 1.0
        RState.Fog_Start = 100
        RState.Fog_End = 1500
        RState.Fog_Property(eFog_Mode.LINEAR, eFog_Type.Table)
        RState.Fog_Color = Color.SkyBlue
        RState.Fog_Enable = True

        'on créé notre camera
        myCamera = New Scene.TZCamera("myCamera")

        'on la positionne au coordonnée X=1024, Y=150, Z=1024  
        myCamera.Position = New Vector3(1024, 150, 1024)
        myCamera.LookAt(myCamera.Position + New Vector3(1, -0.25, 0))

        'camera en mode Fps
        myCamera.Set_Mode(eCameraMode.First_Person)
        myCamera.Perspective(60, Engine.Viewport_Width / Engine.Viewport_Height, 1, 2000)
               

        Render3D()
    End Sub

    Private Sub Render3D()

        'attend que tout est chargé
        Engine.WaitLoad(False, False, True)

        'debut de la boucle de rendu
        Do While Engine.DoRender
            Application.DoEvents()

            's'il n'y a pas de perte du device on peu commencer le rendu
            If Engine.TestCooperativeLevel Then

                'début du rendu
                Engine.Render_Begin()

                'effacement du backbuffer avec une couleur
                Engine.Render_Clear(Color.SkyBlue)

                'rotation douce de la camera vers la droite
                myCamera.Rotate_Smoothly(0.1, 0, 0)

                'on indique au moteur qu'il faut regarder à partir de notre camera            
                Engine.Camera_Update(myCamera, False)

                'rendu de la skybox
                mySkybox.Render()

                'mise à jours du terrain suivant la position de la camera actuel
                myTerrain.Update()

                'rendu du terrain
                myTerrain.Render()

                'fin du rendu
                Engine.Render_End()

            End If

        Loop

        'on decharge le moteur et tout ces composants interne
        Engine.Dispose()

        Application.Exit()
        Engine = Nothing
    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If Engine.DoRender = True Then
            e.Cancel = True

            'indique au moteur que l'on veut quitter
            Engine.Quit()
            'et quant il est pret il passe la valeur Engine.DoRender à False
        End If
    End Sub
End Class
