﻿'Dans ce troisième tutorial nous allons aussi charger un objet 3d mais à partir
'd'un fichier .X et lui appliquer de la lumière
Imports Tzu_Engine
Imports Tzu_Engine.TZGlobal

Public Class Form1

    Private Engine As TZEngine = TZEngine.GetInstance

    Private myMesh As Mesh.TZMesh
    Private myFloor As Mesh.TZMesh

    Private myCamera As Scene.TZCamera

    Private myLight As Light.TZLight

    Private Deferred As Graphics.TZDeferredRendering
    Private MeshManager As Mesh.TZMeshManager
    Private TexManager As Texture.TZTextureManager

    Private Point_Light() As Light.TZLight

    Private SpotLight As Light.TZLight
    Private myText As Graphics.TZText

    Private Reverse As Boolean
    Private Range_Inc As Single = 120

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialisation du moteur en mode fenêtré   
        Engine.Init(Me.Handle)

        'indique que nous voulons afficher le nombre d'image par seconde
        Engine.Display_FPS = True

        'Active l'instancing pour optimiser le deferred
        Engine.UseInstancing = True

        'on indique le dossier des medias
        Engine.Add_MediaPath(My.Application.Info.DirectoryPath & "\..\..\..\media\")

        'on indique au moteur que les meshs n'ont pas besoin de chercher les lumières qui les atteignent
        Engine.DoFillLight = False

        'récupère l'instance de la class TZDeferredRendering
        Deferred = Graphics.TZDeferredRendering.GetInstance

        'défini la couleur ambiente global
        Deferred.AmbienColor = Get_Color(0.05, 0.05, 0.05, 1)

        MeshManager = Mesh.TZMeshManager.GetInstance
        TexManager = Texture.TZTextureManager.GetInstance

        'on charge notre superbe voiture
        myMesh = New Mesh.TZMesh("car2.x", "myMcLaren")

        'on indique que l'on souhaite rendre visible la lumière spéculaire avec un coefficient de netteté à 15
        myMesh.Material(0).Specular_Enable = True
        myMesh.Material(0).SpecularSharpness = 15
        myMesh.Material(0).Diffuse = Color.White

        myMesh.load()

        'on ajoute un sol sous la voiture avec du normalmapping, parallaxmapping et du specularmapping
        myFloor = MeshManager.Load_Cube(1, 0.2, 1, "danceFloor")

        myFloor.Material(0).DiffuseMap = TexManager.Load("Fieldstone.jpg", "Floor_Diffuse")
        myFloor.Material(0).NormalMap = TexManager.Load("FieldstoneNormal.jpg", "Floor_Normal")
        myFloor.Material(0).HeightMap = TexManager.Load("FieldstoneBump.jpg", "Floor_Height")
        myFloor.Material(0).GlossMap = TexManager.Load("FieldstoneBump.jpg", "Floor_Height")
        myFloor.Material(0).Diffuse = Color.White

        myFloor.Material(0).NormalMap_Enable = True
        myFloor.Material(0).Parallax_Enable = True

        myFloor.Material(0).Specular_Enable = True
        myFloor.Material(0).SpecularSharpness = 25

        myFloor.Scale = New Vector3(1000, 1, 1000)
        myFloor.load()      

        myText = New Graphics.TZText("arial", 12, FontStyle.Regular)
       
        'on créé notre camera
        myCamera = New Scene.TZCamera("myCamera")
        myCamera.Position = New Vector3(-500, 300, 0)
        myCamera.Perspective(75, CSng(Engine.Viewport_Width / Engine.Viewport_Height), 1, -1)
        myCamera.LookAt(myMesh.Position)

        'création de 12 Point light qui tourneront autour de la voiture
        ReDim Point_Light(11)
        For i As Integer = 0 To 11
            Point_Light(i) = New Light.TZLight("point_light_" & i)
            Point_Light(i).Type = eLight_Type.Point
            Point_Light(i).Diffuse = Get_Color(Rnd() * 1, Rnd() * 1, Rnd() * 1, 1)
            Point_Light(i).Intensity = 1
            Point_Light(i).Enable = True
        Next

        'création d'une spotlight d'un angle de 45° dans la même direction que la camera
        SpotLight = New Light.TZLight("myspotlight")
        SpotLight.Type = eLight_Type.Spot
        SpotLight.OuterConeAngle = DegToRad(45)
        SpotLight.InnerConeAngle = DegToRad(30)
        SpotLight.Range = 700
        SpotLight.Falloff = 1
        SpotLight.Diffuse = Color.White

        'sera activé lors du clique de la souris
        SpotLight.Enable = False

        SpotLight.Direction = myCamera.Look
        SpotLight.position = myCamera.Position

        SpotLight.Intensity = 2


        Render3D()
    End Sub

    Private Sub Render3D()

        'attend que tout est chargé
        Engine.WaitLoad(False, False, True)

        myFloor.Position = New Vector3(0, -myMesh.Bounding_Box.Height / 2 - 0.1F, 0)
        Dim Increment As Single

        'debut de la boucle de rendu
        Do While Engine.DoRender
            Application.DoEvents()

            's'il n'y a pas de perte du device on peu commencer le rendu
            If Engine.TestCooperativeLevel Then
                Increment += 1 * Engine.DeltaTime

                'on fait tourner toute les lumière autour de la voiture avec pour rayon la taille de la sa sphère englobante
                PointLight(Point_Light, myMesh.Position + New Vector3(0, myMesh.Bounding_Box.Height / 2, 0), myMesh.Bounding_Sphere.Radius, Increment)

                'on indique au moteur qu'il faut regarder à partir de notre camera   
                Engine.Camera_Update(myCamera, False)

                'début du rendu vers le Gbuffer
                Deferred.GBuffer_Begin()

                'nous faisons tourner notre voiture sur elle même en utilisant
                'DeltaTime afin d'avoir une vitesse constante
                myMesh.Rotate_StepY(50 * Engine.DeltaTime)

                'et on la dessine
                myMesh.Render()

                'pareil pour le sol
                myFloor.Render()

                'fin de rendu du Gbuffer
                Deferred.GBuffer_End()


                'début du rendu
                Engine.Render_Begin()

                'effacement du backbuffer
                Engine.Render_Clear()

                'rendu de la scène éclairé
                Deferred.Render()              

                myText.Write("cliquez pour allumer la spotlight!", New Vector2(0, 32), Color.Yellow)

                'fin du rendu
                Engine.Render_End()

            End If

        Loop

        'on decharge le moteur et tout ces composants interne
        Engine.Dispose()

        Application.Exit()

    End Sub

    Private Sub PointLight(ByVal Light() As Light.TZLight, ByVal Position As Vector3, ByVal Radius As Single, ByVal Increment As Single)

        'inverse l'incrémentation du rayon des lumières s'il est supèrieur à 256 ou inférieur à 120
        If Range_Inc > 256 Then
            Reverse = True
        ElseIf Range_Inc < 120 Then
            Reverse = False
        End If

        'Inversion?
        If Reverse = False Then
            Range_Inc += 11 * Engine.DeltaTime
        Else
            Range_Inc -= 11 * Engine.DeltaTime
        End If

        'cercle divisé par le nombre de lumière afin d'obtenir leurs positions
        Dim Incr_Rad As Single = DegToRad(360.0F / Light.Length)

        For i As Integer = 0 To Light.Length - 1
            Point_Light(i).Range = Range_Inc
            Light(i).position = New Vector3(CSng(Position.X + Math.Cos((i + 1) * Incr_Rad + Increment) * Radius), Position.Y, CSng(Position.Z + Math.Sin((i + 1) * Incr_Rad + Increment) * Radius))
        Next

    End Sub

    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If Engine.DoRender = True Then
            e.Cancel = True

            'indique au moteur que l'on veut quitter
            Engine.Quit()
            'et quant il est pret il passe la valeur Engine.DoRender à False
        End If
    End Sub

    Private Sub Form1_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        SpotLight.Enable = True
    End Sub

    Private Sub Form1_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseUp
        SpotLight.Enable = False
    End Sub
End Class
